#  cuplcodec encodes environmental sensor data into a URL and the reverse.
#
#  https://github.com/cuplsensor/cuplcodec
#
#  Original Author: Malcolm Mackay
#  Email: malcolm@plotsensor.com
#  Website: https://cupl.co.uk
#
#  Copyright (C) 2021. Plotsensor Ltd.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#

import pytest
from wscodec.encoder.pyencoder.instrumented import InstrumentedSampleTRH
from wscodec.decoder import decode

INPUT_SERIAL = 'abcdabcd'
INPUT_TIMEINT = 12
INPUT_SECKEY = 'AAAABBBBCCCCDDDD'
INPUT_STATUSB64 = 'MDAwMDAw'
INPUT_CBUFLENBLKS = 32

PADDING = 2


@pytest.fixture(scope="function",
                params=[{'baseurl': "thehrhsdbsdrd.plotsensor.com", 'secretkey': INPUT_SECKEY},
                        {'baseurl': "plotsensor.com", 'secretkey': INPUT_SECKEY}
                        ])
def instr_sample(request):
    return InstrumentedSampleTRH(baseurl=request.param['baseurl'],
                              serial=INPUT_SERIAL,
                              secretkey=request.param['secretkey'],
                              smplintervalmins=INPUT_TIMEINT)


@pytest.mark.parametrize('n', range(1, 150))
def test_cursorpos(instr_sample, n):
    instr_sample.pushsamples(n)
    encoder_cursorpos = instr_sample.ffimodule.lib.demi_getendmarkerpos()

    # Decode the URL
    par = instr_sample.eepromba.get_url_parsedqs()
    decodedurl = decode(secretkey=instr_sample.secretkey, statb64=par['x'][0], timeintb64=par['t'][0],
                        circb64=par['q'][0], vfmtb64=par['v'][0])

    assert encoder_cursorpos == decodedurl.endmarkerpos



#  cuplcodec encodes environmental sensor data into a URL and the reverse.
#
#  https://github.com/cuplsensor/cuplcodec
#
#  Original Author: Malcolm Mackay
#  Email: malcolm@plotsensor.com
#  Website: https://cupl.co.uk
#
#  Copyright (C) 2021. Plotsensor Ltd.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#

import pytest
from wscodec.decoder import decode


@pytest.fixture(scope="function", params=[1, 10, 100, 1000, 1001])
def instr_sample_populated(instr_sample, request):
    inlist = instr_sample.pushsamples(request.param)

    # Decode the URL
    par = instr_sample.eepromba.get_url_parsedqs()
    decodedurl = decode(secretkey=instr_sample.secretkey, statb64=par['x'][0], timeintb64=par['t'][0],
                        circb64=par['q'][0], vfmtb64=par['v'][0])

    urllist = decodedurl.get_samples_list()
    for d in urllist:
        del d['timestamp']
        del d['rawtemp']
        if 'rawrh' in d:
            del d['rawrh']

    inlist = inlist[:len(urllist)]

    return {'instr_sample': instr_sample, 'inlist': inlist, 'urllist': urllist}


#  cuplcodec encodes environmental sensor data into a URL and the reverse.
#
#  https://github.com/cuplsensor/cuplcodec
#
#  Original Author: Malcolm Mackay
#  Email: malcolm@plotsensor.com
#  Website: https://cupl.co.uk
#
#  Copyright (C) 2021. Plotsensor Ltd.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#

from wscodec.encoder.pyencoder import unitc

ffibuilder = unitc.load('ndef', ['eep', 'base64', 'nvtype'])

if __name__ == "__main__":
    ffibuilder.compile()


#  cuplcodec encodes environmental sensor data into a URL and the reverse.
#
#  https://github.com/cuplsensor/cuplcodec
#
#  Original Author: Malcolm Mackay
#  Email: malcolm@plotsensor.com
#  Website: https://cupl.co.uk
#
#  Copyright (C) 2021. Plotsensor Ltd.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#

# Helpful: https://julien.danjou.info/python-exceptions-guide/

class DecoderError(Exception):
    """Base cupl Decoder error class."""

    def __init__(self, msg="cupl Decoder Error"):
        super().__init__(msg)


class InvalidMajorVersionError(DecoderError):
    def __init__(self, encoderversion, decoderversion, msg=None):
        if msg is None:
            # Set a default error messasge
            msg = "There is a mismatch between the encoder version = {} " \
                  "and decoder version = {} ".format(encoderversion, decoderversion)
        super().__init__(msg)
        self.encoderversion = encoderversion
        self.decoderversion = decoderversion


class InvalidFormatError(DecoderError):
    def __init__(self, circformat, msg=None):
        if msg is None:
            # Set a default error message
            msg = "Invalid circular buffer format = {}.".format(circformat)
        super().__init__(msg)
        self.circformat = circformat


class MessageIntegrityError(DecoderError):
    def __init__(self, calchash, urlhash, msg=None):
        if msg is None:
            # Set a default error message
            msg = "Checksum mismatch. Calculated hash = {}, URL hash = {}".format(calchash, urlhash)
        super().__init__(msg)
        self.calchash = calchash
        self.urlhash = urlhash


class NoCircularBufferError(DecoderError):
    def __init__(self, status, msg=None):
        if msg is None:
            # Set a default error message
            msg = "There is no circular buffer. This is indicative of an error with the system running the encoder. " \
                  "Status = {}".format(status)
        super().__init__(msg)
        self.status = status


class DelimiterNotFoundError(DecoderError):
    def __init__(self, circb64, status, msg=None):
        if msg is None:
            msg = " No delimiting character found in the circular buffer string = {}. The firmware has initialised " \
                  "the encoder but not pushed data. Status = {}".format(circb64, status)
        super().__init__(msg)
        self.circb64 = circb64
        self.status = status



#  cuplcodec encodes environmental sensor data into a URL and the reverse.
#
#  https://github.com/cuplsensor/cuplcodec
#
#  Original Author: Malcolm Mackay
#  Email: malcolm@plotsensor.com
#  Website: https://cupl.co.uk
#
#  Copyright (C) 2021. Plotsensor Ltd.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#

from datetime import datetime
from .instrumented import InstrumentedSampleT, InstrumentedSampleTRH, InstrumentedSample


def encode(format,
           serial,
           secretkey,
           baseurl,
           smplintervalmins,
           resetsalltime,
           batteryadc,
           resetcause,
           usehmac,
           httpsdisable,
           tagerror) -> InstrumentedSample:
    """
    Python-wrapped encoder factory.

    Returns
    --------
    InstrumentedSample
        An object containing a list of timestamped environmental sensor samples.

    """

    encoder = _get_encoder(format)(serial=serial,
                                   secretkey=secretkey,
                                   baseurl=baseurl,
                                   smplintervalmins=smplintervalmins,
                                   resetsalltime=resetsalltime,
                                   batteryadc=batteryadc,
                                   resetcause=resetcause,
                                   usehmac=usehmac,
                                   httpsdisable=httpsdisable,
                                   tagerror=tagerror,
                                   format=format)
    return encoder


def _get_encoder(format: int):
    """
        Parameters
        -----------
        formatcode:
            Value of the codec format field. Specifies which decoder shall be returned.

        Return
        -------
        Decoder class for the given format code.

        """
    encoders = {
        InstrumentedSampleTRH.FORMAT_HDC2021_TRH: InstrumentedSampleTRH,
        InstrumentedSampleT.FORMAT_HDC2021_TEMPONLY: InstrumentedSampleT
    }
    try:
        encoder = encoders[format]
    except KeyError:
        # We need to test sending an unsupported format code.
        encoder = encoders[InstrumentedSampleTRH.FORMAT_HDC2021_TRH]

    return encoder



#  cuplcodec encodes environmental sensor data into a URL and the reverse.
#
#  https://github.com/cuplsensor/cuplcodec
#
#  Original Author: Malcolm Mackay
#  Email: malcolm@plotsensor.com
#  Website: https://cupl.co.uk
#
#  Copyright (C) 2021. Plotsensor Ltd.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#

import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cuplcodec",
    version="2.0.9",
    author="Malcolm Mackay",
    author_email="malcolm@plotsensor.com",
    description="Package for creating and decoding URLs that contain temperature and humidity samples.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cuplsensor/cuplcodec",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
    ],
    include_package_data=True,
    setup_requires=["cffi>=1.0.0"],
    cffi_modules=["wscodec/encoder/pyencoder/sample_builder.py:ffibuilder",
                  "wscodec/encoder/pyencoder/ndef_builder.py:ffibuilder",
                  "wscodec/encoder/pyencoder/demi_builder.py:ffibuilder",
                  "wscodec/encoder/pyencoder/pairhist_builder.py:ffibuilder"],
    install_requires=["cffi>=1.0.0", "ndeflib>=0.3.2"],
)
